-- Author      : Kurapica
-- Create Date : 7/27/2011

---------------------------------------------------------------------------------------------------------------------------------------
--- CodeEditor is using to coding
-- <br><br>inherit <a href=".\MultiLineTextBox.html">MultiLineTextBox</a> For all methods, properties and scriptTypes
-- @name CodeEditor
-- @class table
---------------------------------------------------------------------------------------------------------------------------------------

-- Check Version
local version = 1

if not IGAS:NewAddon("IGAS.Widget.CodeEditor", version) then
	return
end

class "CodeEditor"
	inherit "MultiLineTextBox"

	import "System.Threading"

	script "OnSave"
	script "OnFunctionKey"
	script "OnOperationListChanged"

	local bor = bit.bor
	local band = bit.band
	local tinsert = tinsert
	local tremove = tremove
	local floor = math.floor

	GRN    ='	'
	YEL    ='|cffffff00'
	RED    ='|cffff0000'
	WHT    ='|cffffffff'
	BLU    ='|cff8888ff'
	DRKBLU ='|cff1b0495'
	ORN    ='|cffff9C00'
	GRY    ='|cffA0A0A0'

	_G = _G
	_DBL_CLK_CHK = 0.3
	_WAITTIME = 0.2

	_IndentNone = 0
	_IndentRight = 1
	_IndentLeft = 2
	_IndentBoth = 3

	_TabWidth = 4
	_DefaultColor = System.Widget.ColorType(1, 1, 1, 1)
	_CommentColor = System.Widget.ColorType(0, 1, 0, 1)
	_StringColor = System.Widget.ColorType(0, 1, 0, 1)
	_NumberColor = System.Widget.ColorType(1, 1, 0, 1)
	_InstructionColor = System.Widget.ColorType(1, 0.39, 0.09, 1)
	_EndColor = "|r"

	-- Token
	_Token = {
		UNKNOWN = 0,
		LINEBREAK = 1,
		SPACE = 2,
		OPERATOR = 3,
		LEFTBRACKET = 4,
		RIGHTBRACKET = 5,
		LEFTPAREN = 6,
		RIGHTPAREN = 7,
		LEFTWING = 8,
		RIGHTWING = 9,
		COMMA = 10,
		SEMICOLON = 11,
		COLON = 12,
		HASH = 13,
		NUMBER = 14,
		COLORCODE_START = 15,
		COLORCODE_END = 16,
		COMMENT = 17,
		STRING = 18,
		ASSIGNMENT = 19,
		EQUALITY = 20,
		PERIOD = 21,
		DOUBLEPERIOD = 22,
		TRIPLEPERIOD = 23,
		LT = 24,
		LTE = 25,
		GT = 26,
		GTE = 27,
		NOTEQUAL = 28,
		TILDE = 29,
		IDENTIFIER = 30,
	}

	_WordWrap = {
		[0] = _IndentNone,	-- UNKNOWN
		_IndentNone,		-- LINEBREAK
		_IndentNone,		-- SPACE
		_IndentBoth,		-- OPERATOR
		_IndentNone,		-- LEFTBRACKET
		_IndentNone,		-- RIGHTBRACKET
		_IndentNone,		-- LEFTPAREN
		_IndentNone,		-- RIGHTPAREN
		_IndentNone,		-- LEFTWING
		_IndentNone,		-- RIGHTWING
		_IndentRight,		-- COMMA
		_IndentNone,		-- SEMICOLON
		_IndentNone,		-- COLON
		_IndentLeft,		-- HASH
		_IndentNone,		-- NUMBER
		_IndentNone,		-- COLORCODE_START
		_IndentNone,		-- COLORCODE_END
		_IndentNone,		-- COMMENT
		_IndentNone,		-- STRING
		_IndentBoth,		-- ASSIGNMENT
		_IndentBoth,		-- EQUALITY
		_IndentNone,		-- PERIOD
		_IndentBoth,		-- DOUBLEPERIOD
		_IndentNone,		-- TRIPLEPERIOD
		_IndentBoth,		-- LT
		_IndentBoth,		-- LTE
		_IndentBoth,		-- GT
		_IndentBoth,		-- GTE
		_IndentBoth,		-- NOTEQUAL
		_IndentNone,		-- TILDE
		_IndentNone,		-- IDENTIFIER
	}

	-- Words
	_KeyWord = {
		["and"] = _IndentNone,
		["break"] = _IndentNone,
		["do"] = _IndentRight,
		["else"] = _IndentBoth,
		["elseif"] = _IndentLeft,
		["end"] = _IndentLeft,
		["false"] = _IndentNone,
		["for"] = _IndentNone,
		["function"] = _IndentRight,
		["if"] = _IndentNone,
		["in"] = _IndentNone,
		["local"] = _IndentNone,
		["nil"] = _IndentNone,
		["not"] = _IndentNone,
		["or"] = _IndentNone,
		["repeat"] = _IndentRight,
		["return"] = _IndentNone,
		["then"] = _IndentRight,
		["true"] = _IndentNone,
		["until"] = _IndentLeft,
		["while"] = _IndentNone,
		-- IGAS
		["class"] = _IndentRight,
		["inherit"] = _IndentNone,
		["import"] = _IndentNone,
		["endclass"] = _IndentLeft,
		["script"] = _IndentNone,
		["property"] = _IndentNone,
		["namespace"] = _IndentNone,
		["enum"] = _IndentNone,
		["struct"] = _IndentRight,
		["endstruct"] = _IndentLeft,
	}

	-- Bytes
	_Byte = {
		-- LineBreak
		LINEBREAK_N = strbyte("\n"),
		LINEBREAK_R = strbyte("\r"),

		-- Space
		SPACE = strbyte(" "),
		TAB = strbyte("\t"),

		-- UnderLine
		UNDERLINE = strbyte("_"),

		-- Number
		ZERO = strbyte("0"),
		NINE = strbyte("9"),
		E = strbyte("E"),
		e = strbyte("e"),
		x = strbyte("x"),
		a = strbyte("a"),
		f = strbyte("f"),

		-- String
		SINGLE_QUOTE = strbyte("'"),
		DOUBLE_QUOTE = strbyte('"'),

		-- Operator
		PLUS = strbyte("+"),
		MINUS = strbyte("-"),
		ASTERISK = strbyte("*"),
		SLASH = strbyte("/"),
		PERCENT = strbyte("%"),

		-- Compare
		LESSTHAN = strbyte("<"),
		GREATERTHAN = strbyte(">"),
		EQUALS = strbyte("="),

		-- Parentheses
		LEFTBRACKET = strbyte("["),
		RIGHTBRACKET = strbyte("]"),
		LEFTPAREN = strbyte("("),
		RIGHTPAREN = strbyte(")"),
		LEFTWING = strbyte("{"),
		RIGHTWING = strbyte("}"),

		-- Punctuation
		PERIOD = strbyte("."),
		BACKSLASH = strbyte("\\"),
		COMMA = strbyte(","),
		SEMICOLON = strbyte(";"),
		COLON = strbyte(":"),
		TILDE = strbyte("~"),
		HASH = strbyte("#"),

		-- WOW
		VERTICAL = strbyte("|"),
		r = strbyte("r"),
		c = strbyte("c"),
	}

	-- Special
	_Special = {
		-- LineBreak
		[_Byte.LINEBREAK_N] = _Token.LINEBREAK,
		[_Byte.LINEBREAK_R] = _Token.LINEBREAK,

		-- Space
		[_Byte.SPACE] = _Token.SPACE,
		[_Byte.TAB] = _Token.SPACE,

		-- String
		[_Byte.SINGLE_QUOTE] = -1,
		[_Byte.DOUBLE_QUOTE] = -1,

		-- Operator
		[_Byte.MINUS] = -1,	-- need check
		[_Byte.PLUS] = _Token.OPERATOR,
		[_Byte.SLASH] = _Token.OPERATOR,
		[_Byte.ASTERISK] = _Token.OPERATOR,
		[_Byte.PERCENT] = _Token.OPERATOR,

		-- Compare
		[_Byte.LESSTHAN] = -1,
		[_Byte.GREATERTHAN] = -1,
		[_Byte.EQUALS] = -1,

		-- Parentheses
		[_Byte.LEFTBRACKET] = -1,
		[_Byte.RIGHTBRACKET] = _Token.RIGHTBRACKET,
		[_Byte.LEFTPAREN] = _Token.LEFTPAREN,
		[_Byte.RIGHTPAREN] = _Token.RIGHTPAREN,
		[_Byte.LEFTWING] = _Token.LEFTWING,
		[_Byte.RIGHTWING] = _Token.RIGHTWING,

		-- Punctuation
		[_Byte.PERIOD] = -1,
		[_Byte.COMMA] = _Token.COMMA,
		[_Byte.SEMICOLON] = _Token.SEMICOLON,
		[_Byte.COLON] = _Token.COLON,
		[_Byte.TILDE] = -1,
		[_Byte.HASH] = _Token.HASH,

		-- WOW
		[_Byte.VERTICAL] = -1,
	}

	-- Operation
	_Operation = {
		CHANGE_CURSOR = 1,
		INPUTCHAR = 2,
		INPUTTAB = 3,
		DELETE = 4,
		BACKSPACE = 5,
		ENTER = 6,
		PASTE = 7,
		CUT = 8,
	}

	_KEY_OPER = {
		PAGEUP = _Operation.CHANGE_CURSOR,
		PAGEDOWN = _Operation.CHANGE_CURSOR,
		HOME = _Operation.CHANGE_CURSOR,
		END = _Operation.CHANGE_CURSOR,
		UP = _Operation.CHANGE_CURSOR,
		DOWN = _Operation.CHANGE_CURSOR,
		RIGHT = _Operation.CHANGE_CURSOR,
		LEFT = _Operation.CHANGE_CURSOR,
		TAB = _Operation.INPUTTAB,
		DELETE = _Operation.DELETE,
		BACKSPACE = _Operation.BACKSPACE,
		ENTER = _Operation.ENTER,
	}

	-- SkipKey
	_SkipKey = {
		-- Control keys
		LALT = true,
		LCTRL = true,
		LSHIFT = true,
		RALT = true,
		RCTRL = true,
		RSHIFT = true,
		-- other nouse keys
		ESCAPE = true,
		CAPSLOCK = true,
		PRINTSCREEN = true,
		INSERT = true,
		UNKNOWN = true,
	}

	------------------------------------------------------
	-- Key Scanner
	------------------------------------------------------
	_KeyScan = Frame("IGAS_CodeEditor_KeyScan", IGAS.WorldFrame)
	_KeyScan:SetPropagateKeyboardInput(true)
	_KeyScan.KeyboardEnabled = true
	_KeyScan.FrameStrata = "TOOLTIP"
	_KeyScan.Visible = false

	------------------------------------------------------
	-- LongPress check Timer
	------------------------------------------------------
	_CheckTimer = Timer("LongPressCheckTimer", _KeyScan)

	------------------------------------------------------
	-- Code smart helper
	------------------------------------------------------
	_CodeSmartHelper = EditBox("IGAS_CodeEditor_CodeHelper", IGAS.WorldFrame)
	_CodeSmartHelper:SetBackdropColor(0, 0, 0, 1)
	_CodeSmartHelper:SetTextInsets(0, 0, 0, 0)
	_CodeSmartHelper.JustifyH = "LEFT"
	_CodeSmartHelper.JustifyV = "MIDDLE"
	_CodeSmartHelper.FontObject = "GameFontNormalSmall"
	_CodeSmartHelper.FrameStrata = "TOOLTIP"
	_CodeSmartHelper.MouseEnabled = true
	_CodeSmartHelper.AutoFocus = true
	_CodeSmartHelper.Width = 180
	_CodeSmartHelper.Height = 26
	_CodeSmartHelper.Visible = false

	_List = List("List", _CodeSmartHelper)
	_List.FrameStrata = "TOOLTIP"
	_List.DisplayItemCount = 8

	------------------------------------------------------
	-- Code smart helper
	------------------------------------------------------
	_Thread = System.Threading.Thread()
	
    -- Scripts
	local floor = math.floor
	local strlen = string.len
	local strformat = string.format
	local strfind = string.find
	local strsub = string.sub
	local strbyte = string.byte
	local strchar = string.char
	local strrep = string.rep
	local strsub = string.gsub
	local strtrim = strtrim or function(s)
	  return (s:gsub("^%s*(.-)%s*$", "%1")) or ""
	end
	local tblconcat = table.concat

	local function strinsert(str, pos, InsertStr)
		return str:sub(1, pos)..InsertStr..str:sub(pos + 1, -1)
	end

	local function strdelete(str, startPos, endPos)
		return str:sub(1, startPos - 1)..str:sub(endPos + 1)
	end

	local function ReplaceBlock(str, startp, endp, replace)
		return str:sub(1, startp - 1) .. replace .. str:sub(endp + 1, -1)
	end

	local function AdjustCursorPosition(self, pos)
		self.__OldCursorPosition = pos
		self.CursorPosition = pos
		return self:HighlightText(pos, pos)
	end

	-- Token
	local function nextNumber(str, pos, noPeriod, cursorPos, trueWord, newPos)
		pos = pos or 1
		newPos = newPos or 0
		cursorPos = cursorPos or 0

		-- just match, don't care error
		local e = 0
		local startPos = pos

		local byte = strbyte(str, pos)

		-- Number
		while true do
			if byte == _Byte.VERTICAL then
				-- handle the color code
				pos = pos + 1
				byte = strbyte(str, pos)

				if byte == _Byte.c then
					trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
					pos = pos + 9
				elseif byte == _Byte.r then
					trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
					pos = pos + 1
				else
					pos = pos - 1
					break
				end

				startPos = pos
				byte = strbyte(str, pos)
			else
				if not byte then
					break
				elseif byte >= _Byte.ZERO and byte <= _Byte.NINE then
					if e == 1 then e = 2 end
				elseif byte == _Byte.E or byte == _Byte.e then
					if e == 0 then
						e = 1
					end
				elseif byte >= _Byte.a and byte <= _Byte.f then
					if e == 1 then e = 2 end
				elseif not noPeriod and e == 0 and byte == _Byte.PERIOD then
					-- '.' only work before 'e'
					noPeriod = true
				elseif e == 1 and (byte == _Byte.MINUS or byte == _Byte.PLUS) then
					e = 2
				else
					break
				end

				if pos <= cursorPos then
					newPos = newPos + 1
				end

				pos = pos + 1
				byte = strbyte(str, pos)
			end
		end

		trueWord = trueWord and trueWord .. str:sub(startPos, pos - 1)

		return pos, trueWord, newPos
	end

	local function nextComment(str, pos, cursorPos, trueWord, newPos)
		pos = pos or 1
		newPos = newPos or 0
		cursorPos = cursorPos or 0

		local markLen = 0
		local dblBrak = false
		local startPos = pos

		local byte = strbyte(str, pos)

		while byte == _Byte.VERTICAL do
			-- handle the color code
			pos = pos + 1
			byte = strbyte(str, pos)

			if byte == _Byte.c then
				trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
				pos = pos + 9
			elseif byte == _Byte.r then
				trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
				pos = pos + 1
			else
				pos = pos - 1
				break
			end

			startPos = pos
			byte = strbyte(str, pos)
		end

		if byte == _Byte.LEFTBRACKET then
			if pos <= cursorPos then
				newPos = newPos + 1
			end

			pos = pos + 1
			byte = strbyte(str, pos)

			while true do
				if byte == _Byte.VERTICAL then
					-- handle the color code
					pos = pos + 1
					byte = strbyte(str, pos)

					if byte == _Byte.c then
						trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
						pos = pos + 9
					elseif byte == _Byte.r then
						trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
						pos = pos + 1
					else
						pos = pos - 1
						break
					end

					startPos = pos
					byte = strbyte(str, pos)
				else
					if not byte then
						break
					elseif byte == _Byte.EQUALS then
						markLen = markLen + 1
					elseif byte == _Byte.LEFTBRACKET then
						dblBrak = true
					else
						break
					end

					if pos <= cursorPos then
						newPos = newPos + 1
					end

					pos = pos + 1
					byte = strbyte(str, pos)

					if dblBrak then
						break
					end
				end
			end
		end

		if dblBrak then
			--[==[...]==]
			while true do
				if byte == _Byte.VERTICAL then
					-- handle the color code
					pos = pos + 1
					byte = strbyte(str, pos)

					if byte == _Byte.c then
						trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
						pos = pos + 9
					elseif byte == _Byte.r then
						trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
						pos = pos + 1
					else
						pos = pos - 1
						break
					end

					startPos = pos
					byte = strbyte(str, pos)
				else
					if not byte then
						break
					elseif byte == _Byte.RIGHTBRACKET then
						local len = 0

						if pos <= cursorPos then
							newPos = newPos + 1
						end

						pos = pos + 1
						byte = strbyte(str, pos)

						while true do
							if byte == _Byte.VERTICAL then
								-- handle the color code
								pos = pos + 1
								byte = strbyte(str, pos)

								if byte == _Byte.c then
									trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
									pos = pos + 9
								elseif byte == _Byte.r then
									trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
									pos = pos + 1
								else
									pos = pos - 1
									break
								end

								startPos = pos
								byte = strbyte(str, pos)
							else
								if byte == _Byte.EQUALS then
									len = len + 1
								else
									break
								end

								if pos <= cursorPos then
									newPos = newPos + 1
								end

								pos = pos + 1
								byte = strbyte(str, pos)
							end
						end

						if not byte then
							break
						elseif len == markLen and byte == _Byte.RIGHTBRACKET then
							if pos <= cursorPos then
								newPos = newPos + 1
							end

							pos = pos + 1
							byte = strbyte(str, pos)

							break
						end
					end

					if pos <= cursorPos then
						newPos = newPos + 1
					end

					pos = pos + 1
					byte = strbyte(str, pos)
				end
			end
		else
			--...
			while true do
				if byte == _Byte.VERTICAL then
					-- handle the color code
					pos = pos + 1
					byte = strbyte(str, pos)

					if byte == _Byte.c then
						trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
						pos = pos + 9
					elseif byte == _Byte.r then
						trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
						pos = pos + 1
					else
						pos = pos - 1
						break
					end

					startPos = pos
					byte = strbyte(str, pos)
				else
					if not byte or byte == _Byte.LINEBREAK_N or byte == _Byte.LINEBREAK_R then
						break
					end

					if pos <= cursorPos then
						newPos = newPos + 1
					end

					pos = pos + 1
					byte = strbyte(str, pos)
				end
			end
		end

		trueWord = trueWord and trueWord .. str:sub(startPos, pos - 1)

		return pos, trueWord, newPos
	end

	local function nextString(str, pos, mark, cursorPos, trueWord, newPos)
		pos = pos or 1
		cursorPos = cursorPos or 0
		newPos = newPos or 0

		local byte
		local preEscape = false

		local startPos = pos

		if pos <= cursorPos then
			newPos = newPos + 1
		end

		pos = pos + 1
		byte = strbyte(str, pos)

		while true do
			if byte == _Byte.VERTICAL then
				-- handle the color code
				pos = pos + 1
				byte = strbyte(str, pos)

				if byte == _Byte.c then
					trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
					pos = pos + 9
				elseif byte == _Byte.r then
					trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
					pos = pos + 1
				else
					pos = pos - 1
					break
				end

				startPos = pos
				byte = strbyte(str, pos)
			else
				if not byte then
					break
				end

				if not preEscape and byte == mark then
					if pos <= cursorPos then
						newPos = newPos + 1
					end

					pos = pos + 1
					break
				end

				if byte == _Byte.BACKSLASH then
					preEscape = not preEscape
				else
					preEscape = false
				end

				if pos <= cursorPos then
					newPos = newPos + 1
				end

				pos = pos + 1
				byte = strbyte(str, pos)
			end
		end

		trueWord = trueWord and trueWord .. str:sub(startPos, pos - 1)

		return pos, trueWord, newPos
	end

	local function nextIdentifier(str, pos, cursorPos, trueWord, newPos)
		pos = pos or 1
		cursorPos = cursorPos or 0
		newPos = newPos or 0

		local byte
		local startPos = pos

		if pos <= cursorPos then
			newPos = newPos + 1
		end

		pos = pos + 1
		byte = strbyte(str, pos)

		while true do
			if byte == _Byte.VERTICAL then
				-- handle the color code
				pos = pos + 1
				byte = strbyte(str, pos)

				if byte == _Byte.c then
					trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
					pos = pos + 9
				elseif byte == _Byte.r then
					trueWord = trueWord and trueWord .. str:sub(startPos, pos - 2)
					pos = pos + 1
				else
					pos = pos - 1
					break
				end

				startPos = pos
				byte = strbyte(str, pos)
			else
				if not byte then
					break
				elseif byte == _Byte.SPACE or byte == _Byte.TAB then
					break
				elseif _Special[byte] then
					break
				end

				if pos <= cursorPos then
					newPos = newPos + 1
				end

				pos = pos + 1
				byte = strbyte(str, pos)
			end
		end

		trueWord = trueWord and trueWord .. str:sub(startPos, pos - 1)

		return pos, trueWord, newPos
	end

	local function nextToken(str, pos, cursorPos, needTrueWord)
		pos = pos or 1
		cursorPos = cursorPos or 0

		local byte = strbyte(str, pos)
		local start

		if not byte then return nil, pos end

		start = pos

		-- Space
		if byte == _Byte.SPACE or byte == _Byte.TAB then
			while true do
				pos = pos + 1
				byte = strbyte(str, pos)

				if not byte or (byte ~= _Byte.SPACE and byte ~= _Byte.TAB) then
					return _Token.SPACE, pos, needTrueWord and str:sub(start, pos - 1)
				end
			end
		end

		-- Special character
		if _Special[byte] then
			if _Special[byte] >= 0 then
				return _Special[byte], pos + 1, needTrueWord and str:sub(start, pos)
			elseif _Special[byte] == -1 then
				if byte == _Byte.VERTICAL then
					-- '|'
					pos = pos + 1
					byte = strbyte(str, pos)

					if byte == _Byte.c then
						--[[for i = pos + 1, pos + 8 do
							byte = strbyte(str, i)

							if i <= pos + 2 then
								if byte ~= _Byte.f then
									return _Token.UNKNOWN, pos
								end
							else
								if not ( ( byte >= _Byte.ZERO and byte <= _Byte.NINE ) or ( byte >= _Byte.a and byte <= _Byte.f ) ) then
									return  _Token.UNKNOWN, pos
								end
							end
						end--]]

						-- mark as '|cff20ff20'
						return _Token.COLORCODE_START, pos + 9
					elseif byte == _Byte.r then
						-- mark '|r'
						return _Token.COLORCODE_END, pos + 1
					else
						-- don't know
						return _Token.UNKNOWN, pos
					end
				elseif byte == _Byte.MINUS then
					-- '-'
					pos = pos + 1
					byte = strbyte(str, pos)

					while byte == _Byte.VERTICAL do
						-- handle the color code
						pos = pos + 1
						byte = strbyte(str, pos)

						if byte == _Byte.c then
							pos = pos + 9
						elseif byte == _Byte.r then
							pos = pos + 1
						else
							pos = pos - 1
							break
						end

						byte = strbyte(str, pos)
					end

					if byte == _Byte.MINUS then
						-- '--'
						return _Token.COMMENT, nextComment(str, pos + 1, cursorPos, needTrueWord and "--", 2)
					else
						-- '-'
						return _Token.OPERATOR, pos, "-", 1
					end
				elseif byte == _Byte.SINGLE_QUOTE or byte == _Byte.DOUBLE_QUOTE then
					-- ' || "
					return _Token.STRING, nextString(str, pos, byte, cursorPos, needTrueWord and "")
				elseif byte == _Byte.LEFTBRACKET then
					local chkPos = pos
					local dblBrak = false

					-- '['
					pos = pos + 1
					byte = strbyte(str, pos)

					while true do
						if byte == _Byte.VERTICAL then
							-- handle the color code
							pos = pos + 1
							byte = strbyte(str, pos)

							if byte == _Byte.c then
								pos = pos + 9
							elseif byte == _Byte.r then
								pos = pos + 1
							else
								pos = pos - 1
								break
							end

							byte = strbyte(str, pos)
						else
							if not byte then
								break
							elseif byte == _Byte.EQUALS then
							elseif byte == _Byte.LEFTBRACKET then
								dblBrak = true
								break
							else
								break
							end

							pos = pos + 1
							byte = strbyte(str, pos)
						end
					end

					if dblBrak then
						return _Token.STRING, nextComment(str, chkPos, cursorPos, needTrueWord and "")
					else
						return _Token.LEFTBRACKET, chkPos + 1, "[", 1
					end
				elseif byte == _Byte.EQUALS then
					-- '='
					pos = pos + 1
					byte = strbyte(str, pos)

					while byte == _Byte.VERTICAL do
						-- handle the color code
						pos = pos + 1
						byte = strbyte(str, pos)

						if byte == _Byte.c then
							pos = pos + 9
						elseif byte == _Byte.r then
							pos = pos + 1
						else
							pos = pos - 1
							break
						end

						byte = strbyte(str, pos)
					end

					if byte == _Byte.EQUALS then
						return _Token.EQUALITY, pos + 1, "==", 2
					else
						return _Token.ASSIGNMENT, pos, "=", 1
					end
				elseif byte == _Byte.PERIOD then
					-- '.'
					pos = pos + 1
					byte = strbyte(str, pos)

					while byte == _Byte.VERTICAL do
						-- handle the color code
						pos = pos + 1
						byte = strbyte(str, pos)

						if byte == _Byte.c then
							pos = pos + 9
						elseif byte == _Byte.r then
							pos = pos + 1
						else
							pos = pos - 1
							break
						end

						byte = strbyte(str, pos)
					end

					if not byte then
						return _Token.PERIOD, pos, ".", 1
					elseif byte == _Byte.PERIOD then
						pos = pos + 1
						byte = strbyte(str, pos)

						while byte == _Byte.VERTICAL do
							-- handle the color code
							pos = pos + 1
							byte = strbyte(str, pos)

							if byte == _Byte.c then
								pos = pos + 9
							elseif byte == _Byte.r then
								pos = pos + 1
							else
								pos = pos - 1
								break
							end

							byte = strbyte(str, pos)
						end

						if byte == _Byte.PERIOD then
							return _Token.TRIPLEPERIOD, pos + 1, "...", 3
						else
							return _Token.DOUBLEPERIOD, pos, "..", 2
						end
					elseif byte >= _Byte.ZERO and byte <= _Byte.NINE then
						return _Token.NUMBER, nextNumber(str, pos, true, cursorPos, needTrueWord and ".", 1)
					else
						return _Token.PERIOD, pos, ".", 1
					end
				elseif byte == _Byte.LESSTHAN then
					-- '<'
					pos = pos + 1
					byte = strbyte(str, pos)

					while byte == _Byte.VERTICAL do
						-- handle the color code
						pos = pos + 1
						byte = strbyte(str, pos)

						if byte == _Byte.c then
							pos = pos + 9
						elseif byte == _Byte.r then
							pos = pos + 1
						else
							pos = pos - 1
							break
						end

						byte = strbyte(str, pos)
					end

					if byte == _Byte.EQUALS then
						return _Token.LTE, pos + 1, "<=", 2
					else
						return _Token.LT, pos, "<", 1
					end
				elseif byte == _Byte.GREATERTHAN then
					-- '>'
					pos = pos + 1
					byte = strbyte(str, pos)

					while byte == _Byte.VERTICAL do
						-- handle the color code
						pos = pos + 1
						byte = strbyte(str, pos)

						if byte == _Byte.c then
							pos = pos + 9
						elseif byte == _Byte.r then
							pos = pos + 1
						else
							pos = pos - 1
							break
						end

						byte = strbyte(str, pos)
					end

					if byte == _Byte.EQUALS then
						return _Token.GTE, pos + 1, ">=", 2
					else
						return _Token.GT, pos, ">", 1
					end
				elseif byte == _Byte.TILDE then
					-- '~'
					pos = pos + 1
					byte = strbyte(str, pos)

					while byte == _Byte.VERTICAL do
						-- handle the color code
						pos = pos + 1
						byte = strbyte(str, pos)

						if byte == _Byte.c then
							pos = pos + 9
						elseif byte == _Byte.r then
							pos = pos + 1
						else
							pos = pos - 1
							break
						end

						byte = strbyte(str, pos)
					end

					if byte == _Byte.EQUALS then
						return _Token.NOTEQUAL, pos + 1, "~=", 2
					else
						return _Token.TILDE, pos, "~", 1
					end
				else
					return _Token.UNKNOWN, pos
				end
			end
		end

		-- Number
		if byte >= _Byte.ZERO and byte <= _Byte.NINE then
			return _Token.NUMBER, nextNumber(str, pos, nil, cursorPos, needTrueWord and "")
		end

		-- Identifier
		return _Token.IDENTIFIER, nextIdentifier(str, pos, cursorPos, needTrueWord and "")
	end

	-- Color
	local function FormatColor(str, colorTable, cursorPos)
		local pos = 1

		local token
		local content = {}
		local nextPos
		local trueWord
		local word
		local newPos

		local defaultColor = colorTable.DefaultColor.code
		local commentColor = colorTable.CommentColor.code
		local stringColor = colorTable.StringColor.code
		local numberColor = colorTable.NumberColor.code
		local instructionColor = colorTable.InstructionColor.code

		cursorPos = cursorPos or 0

		local chkLength = 0
		local newCurPos = 0

		while true do
			token, nextPos, trueWord, newPos = nextToken(str, pos, cursorPos, true)

			if not token then
				break
			end

			word = trueWord or str:sub(pos, nextPos - 1)
			newPos = newPos or word:len()

			if token == _Token.COLORCODE_START or token == _Token.COLORCODE_END then
				-- clear prev colorcode
				tinsert(content, "")
			elseif token == _Token.IDENTIFIER then
				if _KeyWord[word] then
					tinsert(content, instructionColor .. word .. _EndColor)
				else
					tinsert(content, defaultColor .. word .. _EndColor)
				end
			elseif token == _Token.NUMBER then
				tinsert(content, numberColor .. word .. _EndColor)
			elseif token == _Token.STRING then
				tinsert(content, stringColor .. word .. _EndColor)
			elseif token == _Token.COMMENT then
				tinsert(content, commentColor .. word .. _EndColor)
			else
				tinsert(content, word)
			end

			-- Check cursor position
			if chkLength < cursorPos then
				chkLength = chkLength + nextPos - pos

				if chkLength >= cursorPos then
					if content[#content]:len() > 0 and strbyte(content[#content], 1) == _Byte.VERTICAL then
						newCurPos = newCurPos + newPos + 10
					else
						newCurPos = newCurPos + newPos
					end
				else
					newCurPos = newCurPos + content[#content]:len()
				end
			end

			pos = nextPos
		end

		return tblconcat(content), newCurPos
	end

	-- Indent
	local function FormatIndent(str, tab)
		local pos = 1

		local token
		local content = {}
		local indent = 0
		local nextPos
		local word
		local rightSpace = false
		local index
		local prevIndent = 0
		local startIndent = 0
		local prevToken
		local trueWord

		while true do
			prevToken = token
			token, nextPos, trueWord = nextToken(str, pos, nil, true)

			if not token then
				break
			end

			word = str:sub(pos, nextPos - 1)
			trueWord = trueWord or word

			if token == _Token.LEFTWING then
				indent = indent + 1
				startIndent = startIndent + 1
				tinsert(content, word)
				rightSpace = false
			elseif token == _Token.RIGHTWING then
				indent = indent - 1
				if startIndent > 0 then
					startIndent = startIndent - 1
				else
					prevIndent = prevIndent + 1
				end
				index = #content
				if content[index] == strrep(" ", tab * (indent+1)) then
					content[index] = strrep(" ", tab * indent)
				end
				tinsert(content, word)
				rightSpace = false
			elseif token == _Token.LINEBREAK then
				if rightSpace then
					index = #content
					content[index] = content[index]:gsub("^(.-)%s*$", "%1")
					if content[index] == "" then
						content[index] = strrep(" ", tab * indent)
						-- tremove(content, index)
					end
				end
				tinsert(content, word)
				tinsert(content, strrep(" ", tab * indent))
				rightSpace = true
			elseif token == _Token.SPACE then
				if not rightSpace then
					tinsert(content, " ")
				end
				rightSpace = true
			elseif token == _Token.IDENTIFIER then
				if _KeyWord[trueWord] then
					if _KeyWord[trueWord] == _IndentNone then
						indent = indent
					elseif _KeyWord[trueWord] == _IndentRight then
						indent = indent + 1
						startIndent = startIndent + 1
					elseif _KeyWord[trueWord] == _IndentLeft then
						indent = indent - 1
						if startIndent > 0 then
							startIndent = startIndent - 1
						else
							prevIndent = prevIndent + 1
						end

						if prevToken == _Token.COLORCODE_START then
							index = #content - 1
						else
							index = #content
						end

						if content[index] == strrep(" ", tab * (indent+1)) then
							content[index] = strrep(" ", tab * indent)
						end
					elseif _KeyWord[trueWord] == _IndentBoth then
						indent = indent
						if startIndent == 0 then
							prevIndent = prevIndent + 1
							startIndent = startIndent + 1
						end

						if prevToken == _Token.COLORCODE_START then
							index = #content - 1
						else
							index = #content
						end

						if content[index] == strrep(" ", tab * indent) then
							content[index] = strrep(" ", tab * (indent-1))
						end
					end
					tinsert(content, word)
				else
					tinsert(content, word)
				end
				rightSpace = false
			elseif _WordWrap[token] == _IndentNone then
				tinsert(content, word)
				rightSpace = false
			elseif _WordWrap[token] == _IndentRight then
				tinsert(content, word .. " ")
				rightSpace = true
			elseif _WordWrap[token] == _IndentLeft then
				if rightSpace then
					tinsert(content, word)
				else
					tinsert(content, " " .. word)
				end
				rightSpace = false
			elseif _WordWrap[token] == _IndentBoth then
				if rightSpace then
					tinsert(content, word .. " ")
				else
					tinsert(content, " " .. word .. " ")
				end
				rightSpace = true
			else
				tinsert(content, word)
				rightSpace = false
			end

			pos = nextPos
		end

		str = tblconcat(content)

		wipe(content)

		return str, indent, prevIndent
	end

	_IndentFunc = _IndentFunc or {}
	_ShiftIndentFunc = _ShiftIndentFunc or {}

	do
		setmetatable(_IndentFunc, {
			__index = function(self, key)
				if tonumber(key) then
					local tab = floor(tonumber(key))

					if tab > 0 then
						if not rawget(self, key) then
							rawset(self, key, function(str)
								return strrep(" ", tab) .. str
							end)
						end

						return rawget(self, key)
					end
				end
			end,
		})

		setmetatable(_ShiftIndentFunc, {
			__index = function(self, key)
				if tonumber(key) then
					local tab = floor(tonumber(key))

					if tab > 0 then
						if not rawget(self, key) then
							rawset(self, key, function(str)
								local _, len = str:find("^%s+")

								if len and len > 0 then
									return strrep(" ", len - tab) .. str:sub(len + 1, -1)
								end
							end)
						end

						return rawget(self, key)
					end
				end
			end,
		})

		wipe(_IndentFunc)
		wipe(_ShiftIndentFunc)
	end

	-- Edit function
	local function GetLines(str, startp, endp)
		local byte

		endp = (endp and (endp + 1)) or startp + 1

		-- get prev LineBreak
		while startp > 0 do
			byte = strbyte(str, startp)

			if not byte or byte == _Byte.LINEBREAK_N or byte == _Byte.LINEBREAK_R then
				break
			end

			startp = startp - 1
		end

		startp = startp + 1

		-- get next LineBreak
		while true do
			byte = strbyte(str, endp)

			if not byte or byte == _Byte.LINEBREAK_N or byte == _Byte.LINEBREAK_R then
				break
			end

			endp = endp + 1
		end

		endp = endp - 1

		-- get block
		return startp, endp, str:sub(startp, endp)
	end

	local function GetLinesByReturn(str, startp, returnCnt)
		local byte
		local handledReturn = 0
		local endp = startp + 1

		returnCnt = returnCnt or 0

		-- get prev LineBreak
		while startp > 0 do
			byte = strbyte(str, startp)

			if not byte or byte == _Byte.LINEBREAK_N or byte == _Byte.LINEBREAK_R then
				break
			end

			startp = startp - 1
		end

		startp = startp + 1

		-- get next LineBreak
		while true do
			byte = strbyte(str, endp)

			if not byte then
				break
			elseif byte == _Byte.LINEBREAK_N or byte == _Byte.LINEBREAK_R then
				returnCnt = returnCnt - 1

				if returnCnt < 0 then
					break
				end

				handledReturn = handledReturn + 1
			end

			endp = endp + 1
		end

		endp = endp - 1

		-- get block
		return startp, endp, str:sub(startp, endp), handledReturn
	end

	local function GetPrevLinesByReturn(str, startp, returnCnt)
		local byte
		local handledReturn = 0
		local endp = startp + 1

		returnCnt = returnCnt or 0

		-- get prev LineBreak
		while true do
			byte = strbyte(str, endp)

			if not byte or byte == _Byte.LINEBREAK_N or byte == _Byte.LINEBREAK_R then
				break
			end

			endp = endp + 1
		end

		endp = endp - 1

		-- get prev LineBreak
		while startp > 0 do
			byte = strbyte(str, startp)

			if not byte then
				break
			elseif byte == _Byte.LINEBREAK_N or byte == _Byte.LINEBREAK_R then
				returnCnt = returnCnt - 1

				if returnCnt < 0 then
					break
				end

				handledReturn = handledReturn + 1
			end

			startp = startp - 1
		end

		startp = startp + 1

		-- get block
		return startp, endp, str:sub(startp, endp), handledReturn
	end

	local function GetOffsetByCursorPos(str, startp, cursorPos)
		if not cursorPos or cursorPos < 0 then
			return 0
		end

		startp = startp or cursorPos

		while startp > 0 do
			byte = strbyte(str, startp)

			if not byte or byte == _Byte.LINEBREAK_N or byte == _Byte.LINEBREAK_R then
				break
			end

			startp = startp - 1
		end

		startp = startp + 1

		local byte = strbyte(str, startp)
		local byteCnt = 0

		while startp <= cursorPos do
			if byte == _Byte.VERTICAL then
				-- handle the color code
				startp = startp + 1
				byte = strbyte(str, startp)

				if byte == _Byte.c then
					startp = startp + 9
				elseif byte == _Byte.r then
					startp = startp + 1
				end

				byte = strbyte(str, startp)
			else
				if not byte or byte == _Byte.LINEBREAK_N or byte == _Byte.LINEBREAK_R then
					break
				end

				byteCnt = byteCnt + 1
				startp = startp + 1
				byte = strbyte(str, startp)
			end
		end

		return byteCnt
	end

	local function GetCursorPosByOffset(str, startp, offset)
		startp = startp or 1
		offset = offset or 0

		while startp > 0 do
			byte = strbyte(str, startp)

			if not byte or byte == _Byte.LINEBREAK_N or byte == _Byte.LINEBREAK_R then
				break
			end

			startp = startp - 1
		end

		local byte
		local byteCnt = 0

		while byteCnt < offset do
			startp  = startp  + 1
			byte = strbyte(str, startp)

			if byte == _Byte.VERTICAL then
				-- handle the color code
				startp = startp + 1
				byte = strbyte(str, startp)

				if byte == _Byte.c then
					startp = startp + 8
				end
			else
				if not byte or byte == _Byte.LINEBREAK_N or byte == _Byte.LINEBREAK_R then
					startp = startp - 1
					break
				end

				byteCnt = byteCnt + 1
			end
		end

		return startp
	end

	local function RemoveColor(str, cursorPos)
		local pos = 1

		local token
		local content = {}
		local nextPos
		local trueWord
		local word

		cursorPos = cursorPos or 0

		local chkLength = 0
		local newCurPos = 0

		while true do
			token, nextPos, trueWord = nextToken(str, pos, nil, true)

			if not token then
				break
			end

			word = trueWord or str:sub(pos, nextPos - 1)

			if token == _Token.COLORCODE_START or token == _Token.COLORCODE_END then
				-- clear prev colorcode
				tinsert(content, "")
			else
				tinsert(content, word)
			end

			-- Check cursor position
			if chkLength < cursorPos then
				chkLength = chkLength + nextPos - pos

				if chkLength >= cursorPos then
					newCurPos = newCurPos + cursorPos - (chkLength - word:len())
				else
					newCurPos = newCurPos + content[#content]:len()
				end
			end

			pos = nextPos
		end

		return tblconcat(content), newCurPos
	end

	local function FormatColor4Line(self, startp, endp)
		local cursorPos = self.CursorPosition
		local text = self.FullText
		local byte
		local line

		local commentColor = self.CommentColor.code
		local stringColor = self.StringColor.code

		startp = startp or cursorPos
		endp = endp or cursorPos

		-- Color the line
		startp, endp, line = GetLines(text, startp, endp)

		-- check prev comment
		local preColorPos = startp - 1
		local token, nextPos

		while preColorPos > 0 do
			byte = strbyte(text, preColorPos)

			if byte == _Byte.VERTICAL then
				-- '|'
				byte = strbyte(text, preColorPos + 1)

				if byte == _Byte.c then
					if commentColor == text:sub(preColorPos, preColorPos + 9) or stringColor == text:sub(preColorPos, preColorPos + 9) then
						-- check multi-lines comment or string
						token, nextPos = nextToken(text, preColorPos + 10)

						if token == _Token.COMMENT or token == _Token.STRING then
							if nextPos < startp and nextPos < endp then
								break	-- no need to think about prev multi-lines comment and string
							end
						
							while token and (nextPos <= endp or nextPos <= startp) do
								token, nextPos = nextToken(text, nextPos)
							end
							
							byte = strbyte(text, nextPos)

							if not byte or (nextPos - 1 > endp and nextPos - 1 > startp) then
								line, cursorPos = FormatColor(text:sub(preColorPos, nextPos - 1), self, cursorPos - preColorPos + 1)

								self.FullText = ReplaceBlock(text, preColorPos, nextPos - 1, line)

								cursorPos = preColorPos + cursorPos - 1

								return AdjustCursorPosition(self, cursorPos)
							end

							token, nextPos = nextToken(text, nextPos)

							while token do
								if token == _Token.COLORCODE_START or token == _Token.COLORCODE_END then
									line, cursorPos = FormatColor(text:sub(preColorPos, endp), self, cursorPos - preColorPos + 1)

									self.FullText = ReplaceBlock(text, preColorPos, endp, line)

									cursorPos = preColorPos + cursorPos - 1

									return AdjustCursorPosition(self, cursorPos)
								elseif token == _Token.IDENTIFIER or token == _Token.NUMBER or token == _Token.STRING or token == _Token.COMMENT then
									while token and token ~= _Token.COLORCODE_END do
										token, nextPos = nextToken(text, nextPos)
									end

									line, cursorPos = FormatColor(text:sub(preColorPos, nextPos - 1), self, cursorPos - preColorPos + 1)

									self.FullText = ReplaceBlock(text, preColorPos, nextPos - 1, line)

									cursorPos = preColorPos + cursorPos - 1

									return AdjustCursorPosition(self, cursorPos)
								end

								token, nextPos = nextToken(text, nextPos)
							end
						else
							break
						end
					else
						break
					end
				end
			end

			preColorPos = preColorPos - 1
		end

		nextPos = startp
		token, nextPos = nextToken(text, nextPos)

		while token and (nextPos <= endp or nextPos <= startp) do
			token, nextPos = nextToken(text, nextPos)
		end

		if nextPos - 1 > endp and nextPos - 1 > startp then
			line, cursorPos = FormatColor(text:sub(startp, nextPos - 1), self, cursorPos - startp + 1)

			self.FullText = ReplaceBlock(text, startp, nextPos - 1, line)

			cursorPos = startp + cursorPos - 1

			return AdjustCursorPosition(self, cursorPos)
		end

		while true do
			if not token then
				line, cursorPos = FormatColor(text:sub(startp, endp), self, cursorPos - startp + 1)

				self.FullText = ReplaceBlock(text, startp, endp, line)

				cursorPos = startp + cursorPos - 1

				return AdjustCursorPosition(self, cursorPos)
			elseif token == _Token.COLORCODE_START or token == _Token.COLORCODE_END then
				line, cursorPos = FormatColor(text:sub(startp, endp), self, cursorPos - startp + 1)

				self.FullText = ReplaceBlock(text, startp, endp, line)

				cursorPos = startp + cursorPos - 1

				return AdjustCursorPosition(self, cursorPos)
			elseif token == _Token.IDENTIFIER or token == _Token.NUMBER or token == _Token.STRING or token == _Token.COMMENT then
				while token and token ~= _Token.COLORCODE_END do
					token, nextPos = nextToken(text, nextPos)
				end

				line, cursorPos = FormatColor(text:sub(startp, nextPos - 1), self, cursorPos - startp + 1)

				self.FullText = ReplaceBlock(text, startp, nextPos - 1, line)

				cursorPos = startp + cursorPos - 1

				return AdjustCursorPosition(self, cursorPos)
			end

			token, nextPos = nextToken(text, nextPos)
		end
	end

	local function FormatAll(str, self, noLocal)
		local pos = 1
		local tab = self.TabWidth

		local token
		local content = {}
		local nextPos
		local trueWord
		local word

		local defaultColor = self.DefaultColor.code
		local commentColor = self.CommentColor.code
		local stringColor = self.StringColor.code
		local numberColor = self.NumberColor.code
		local instructionColor = self.InstructionColor.code

		local indent = 0
		local rightSpace = false
		local index
		local prevIndent = 0
		local startIndent = 0
		local prevToken
		
		-- Definition
		local define = self.__Definition

		while true do
			prevToken = token
			token, nextPos, trueWord = nextToken(str, pos, nil, true)

			if not token then
				break
			end

			word = trueWord or str:sub(pos, nextPos - 1)

			if token == _Token.COLORCODE_START or token == _Token.COLORCODE_END then
				-- clear prev colorcode
				tinsert(content, "")
			elseif token == _Token.LEFTWING then
				indent = indent + 1
				startIndent = startIndent + 1
				tinsert(content, word)
				rightSpace = false
			elseif token == _Token.RIGHTWING then
				indent = indent - 1
				if startIndent > 0 then
					startIndent = startIndent - 1
				else
					prevIndent = prevIndent + 1
				end
				index = #content
				if content[index] == strrep(" ", tab * (indent+1)) then
					content[index] = strrep(" ", tab * indent)
				end
				tinsert(content, word)
				rightSpace = false
			elseif token == _Token.LINEBREAK then
				if rightSpace then
					index = #content
					content[index] = content[index]:gsub("^(.-)%s*$", "%1")
					if content[index] == "" then
						content[index] = strrep(" ", tab * indent)
						-- tremove(content, index)
					end
				end
				tinsert(content, word)
				tinsert(content, strrep(" ", tab * indent))
				rightSpace = true
			elseif token == _Token.SPACE then
				if not rightSpace then
					tinsert(content, " ")
				end
				rightSpace = true
			elseif token == _Token.IDENTIFIER then
				if _KeyWord[word] then
					if _KeyWord[word] == _IndentNone then
						indent = indent
					elseif _KeyWord[word] == _IndentRight then
						indent = indent + 1
						startIndent = startIndent + 1
					elseif _KeyWord[word] == _IndentLeft then
						indent = indent - 1
						if startIndent > 0 then
							startIndent = startIndent - 1
						else
							prevIndent = prevIndent + 1
						end

						if prevToken == _Token.COLORCODE_START then
							index = #content - 1
						else
							index = #content
						end

						if content[index] == strrep(" ", tab * (indent+1)) then
							content[index] = strrep(" ", tab * indent)
						end
					elseif _KeyWord[word] == _IndentBoth then
						indent = indent
						if startIndent == 0 then
							prevIndent = prevIndent + 1
							startIndent = startIndent + 1
						end

						if prevToken == _Token.COLORCODE_START then
							index = #content - 1
						else
							index = #content
						end

						if content[index] == strrep(" ", tab * indent) then
							content[index] = strrep(" ", tab * (indent-1))
						end
					end
					tinsert(content, instructionColor .. word .. _EndColor)
				else
					tinsert(content, defaultColor .. word .. _EndColor)
				end
				rightSpace = false
			else
				if token == _Token.NUMBER then
					tinsert(content, numberColor .. word .. _EndColor)
				elseif token == _Token.STRING then
					tinsert(content, stringColor .. word .. _EndColor)
				elseif token == _Token.COMMENT then
					tinsert(content, commentColor .. word .. _EndColor)
				else
					tinsert(content, word)
				end

				if _WordWrap[token] == _IndentNone then
					rightSpace = false
				elseif _WordWrap[token] == _IndentRight then
					content[#content] = content[#content] .. " "
					rightSpace = true
				elseif _WordWrap[token] == _IndentLeft then
					if not rightSpace then
						content[#content] = " " .. content[#content]
					end
					rightSpace = false
				elseif _WordWrap[token] == _IndentBoth then
					if rightSpace then
						content[#content] = content[#content] .. " "
					else
						content[#content] = " " .. content[#content] .. " "
					end
					rightSpace = true
				else
					rightSpace = false
				end
			end

			pos = nextPos
		end

		return tblconcat(content)
	end

	local function SaveOperation(self)
		if not self.__OperationOnLine then
			return
		end

		self.__OperationIndex = self.__OperationIndex + 1
		self.__MaxOperationIndex = self.__OperationIndex

		local index = self.__OperationIndex
		local nowText = self.FullText

		-- Modify some oper var
		if self.__OperationOnLine == _Operation.DELETE then
			local _, oldLineCnt, newLineCnt
			
			_, oldLineCnt = self.__OperationBackUpOnLine:gsub("\n", "\n")
			_, newLineCnt = nowText:gsub("\n", "\n")
			
			_, self.__OperationEndOnLine = GetLinesByReturn(self.__OperationBackUpOnLine, self.__OperationStartOnLine, oldLineCnt - newLineCnt)
		end
		
		if self.__OperationOnLine == _Operation.BACKSPACE then
			local _, oldLineCnt, newLineCnt
			
			_, oldLineCnt = self.__OperationBackUpOnLine:gsub("\n", "\n")
			_, newLineCnt = nowText:gsub("\n", "\n")
			
			self.__OperationStartOnLine = GetPrevLinesByReturn(self.__OperationBackUpOnLine, self.__OperationEndOnLine, oldLineCnt - newLineCnt)
		end

		-- keep operation data
		self.__Operation[index] = self.__OperationOnLine

		self.__OperationBackUp[index] = select(3, GetLines(self.__OperationBackUpOnLine, self.__OperationStartOnLine, self.__OperationEndOnLine))
		self.__OperationStart[index] = self.__OperationStartOnLine
		self.__OperationEnd[index] = self.__OperationEndOnLine

		self.__OperationData[index] = select(3, GetLines(nowText, self.__HighlightTextStart, self.__HighlightTextEnd))
		self.__OperationFinalStart[index] = self.__HighlightTextStart
		self.__OperationFinalEnd[index] = self.__HighlightTextEnd

		-- special operation
		if self.__OperationOnLine == _Operation.ENTER then
			local realStart = GetLines(self.__OperationBackUpOnLine, self.__OperationStartOnLine, self.__OperationEndOnLine)
			if realStart > self.__OperationStartOnLine then
				realStart = self.__OperationStartOnLine
			end
			self.__OperationData[index] = select(3, GetLines(nowText, realStart, self.__HighlightTextEnd))
			self.__OperationFinalStart[index] = realStart
			self.__OperationFinalEnd[index] = self.__HighlightTextEnd
		end

		if self.__OperationOnLine == _Operation.PASTE then
			self.__OperationData[index] = select(3, GetLines(nowText, self.__OperationStartOnLine, self.__HighlightTextEnd))
			self.__OperationFinalStart[index] = self.__OperationStartOnLine
			self.__OperationFinalEnd[index] = self.__HighlightTextEnd
		end

		self.__OperationOnLine = nil
		self.__OperationBackUpOnLine = nil
		self.__OperationStartOnLine = nil
		self.__OperationEndOnLine = nil

		return self:Fire("OnOperationListChanged")
	end

	local function NewOperation(self, oper)
		if self.__OperationOnLine == oper then
			return
		end

		-- save last operation
		SaveOperation(self)

		self.__OperationOnLine = oper

		self.__OperationBackUpOnLine = self.FullText
		self.__OperationStartOnLine = self.__HighlightTextStart
		self.__OperationEndOnLine = self.__HighlightTextEnd
	end

	local function EndPrevKey(self)
		if self.__SKIPCURCHGARROW then
			self.__SKIPCURCHG = nil
			self.__SKIPCURCHGARROW = nil
		end

		self.__BACKSPACE = nil
		self.__DELETE = nil
		self.__INPUTTAB = nil

		self.__InPasting = nil
	end

	local function GetIndex(list, name, sIdx, eIdx)
		if not sIdx then
			if not next(list) then
				return nil
			end
			sIdx = 1
			eIdx = #list
		end
		if sIdx == eIdx then
			if sIdx > 1 then
				return sIdx - 1
			else
				return nil
			end
		end
		local f = floor((sIdx + eIdx) / 2)
		if compare(list[f], name) then
			if not compare(list[f + 1], name) then
				return f
			else
				return GetIndex(list, name, f + 1, eIdx)
			end
		elseif strupper(list[f]) == strupper(name) then
			return GetIndex(list, name, f, f)
		else
			return GetIndex(list, name, sIdx, f)
		end
	end

	local function GetPrevObject(self)
		do
			return nil
		end
		local cursorPos = self.CursorPosition
		local text = self.FullText

		local startp, _, str = GetLines(text, cursorPos)

		str = RemoveColor(str:sub(1, cursorPos - startp))

		local id = str:match("([_%a][_%w%.]*)$")

		if id == "self" then
			while startp > 1 do
				startp, _, str = GetLines(text, startp - 2)

				if str ~= "" then
					str = RemoveColor(str)

					id = str:match("function%s*([_%a][_%w%.]*):")

					if id and id ~= "" then
						break
					end
				end
			end
		end

		local obj
		
		if id and id ~= "" then
			for sub in id:gmatch("[^%.]+") do
				sub = sub and strtrim(sub)
				if not sub or sub =="" then return end

				
			end
		end

		return obj
	end

	local function Search2Next(self)
		if not self.__InSearch then
			return
		end
		
		local text = self.FullText
		
		local startp, endp = text:find(self.__InSearch, self.CursorPosition)
		local startpc, endpc = text:find(self.__InSearch4Color, self.CursorPosition)
		
		if not startp and not startpc then
			startp, endp = text:find(self.__InSearch, 0)
			startpc, endpc = text:find(self.__InSearch4Color, 0)
		end
		
		local s, e
		
		s = startp or startpc
		e = endp or endpc
		
		if startpc and startpc < s then
			s = startpc
		end
		
		if endpc and endpc < e then
			e = endpc
		end
		
		if s and e then
			SaveOperation(self)
		
			AdjustCursorPosition(self, e)

			self:HighlightText(s - 1, e)
		else
			IGAS:MsgBox(L"'%s' can't be found in the content.":format(self.__InSearch))
			
			self:SetFocus()
		end
	end
	
	local function GoLineByNo(self, no)		
		local text = self.FullText
		local lineBreak
		
		if text:find("\n") then
			lineBreak = "\n"
		elseif text:find("\r") then
			lineBreak = "\r"
		else
			lineBreak = false
		end
		
		no = no - 1
		
		local pos = 0
		local newPos
		
		if not lineBreak then
			return
		end
		
		SaveOperation(self)
		
		while no > 0 do
			newPos = text:find(lineBreak, pos + 1)
			
			if newPos then
				no = no - 1
				pos = newPos
			else
				break
			end
		end		
		
		return AdjustCursorPosition(self, pos)
	end
	
	------------------------------------------------------
	-- Method
	------------------------------------------------------
	------------------------------------
	--- Selects all or a portion of the text in the edit box
	-- @name  CodeEditor:HighlightText
	-- @class function
	-- @param start Optional,Character position at which to begin the selection (between 0, for the position before the first character, and  CodeEditor:GetNumLetters(), for the position after the last character); defaults to 0 if not specified
	-- @param end Optional,Character position at which to end the selection; if not specified or if less than start, selects all characters after the start  position; if equal to start, selects nothing and positions the cursor at the start position
	-- @usage  CodeEditor:HighlightText(1, 10)
	------------------------------------
	function HighlightText(self, ...)
		local startp, endp = ...

		if not startp then
			startp = 0
		end

		if not endp then
			endp = self.FullText:len()
		end

		if endp < startp then
			startp, endp = endp, startp
		end

		self.__HighlightTextStart, self.__HighlightTextEnd = startp, endp

		return MultiLineTextBox.HighlightText(self, startp, endp)
	end

	------------------------------------
	--- Sets the edit box's text contents
	-- @name  CodeEditor:SetText
	-- @class function
	-- @param text Text to be placed in the edit box
	-- @usage  CodeEditor:SetText("Hello World")
	------------------------------------
	function SetText(self, str)
		if type(str) ~= "string" then
			error("Usage : CodeEditor:SetText(text) : 'text' - string expected.", 2)
		end

		-- Clear operation history
		wipe(self.__Operation)

		wipe(self.__OperationStart)
		wipe(self.__OperationEnd)
		wipe(self.__OperationBackUp)

		wipe(self.__OperationFinalStart)
		wipe(self.__OperationFinalEnd)
		wipe(self.__OperationData)

		self.__OperationIndex = 0

		-- Clear now operation
		self.__OperationOnLine = nil
		self.__OperationBackUpOnLine = nil
		self.__OperationStartOnLine = nil
		self.__OperationEndOnLine = nil

		-- Format text
		self.FullText = FormatAll(str, self)

		AdjustCursorPosition(self, 0)
	end
	
	------------------------------------
	--- Get the current text contained in the edit box.
	-- @name  CodeEditor:GetText
	-- @class function
	-- @return the current text contained in the edit box
	-- @usage  CodeEditor:GetText()
	------------------------------------
	function GetText(self)
		return RemoveColor(self.FullText)
	end

	------------------------------------
	--- Whether the CodeEditor has operation can be undo.
	-- @name  CodeEditor:CanUndo
	-- @class function
	-- @return true if the CodeEditor can undo operations
	-- @usage  CodeEditor:CanUndo()
	------------------------------------
	function CanUndo(self)
		return self.__OperationIndex > 0
	end

	------------------------------------
	--- Undo the operation.
	-- @name  CodeEditor:Undo
	-- @class function
	-- @usage  CodeEditor:Undo()
	------------------------------------
	function Undo(self)
		SaveOperation(self)

		if self.__OperationIndex > 0 then
			local idx = self.__OperationIndex
			local text = self.FullText
			local startp, endp = GetLines(text, self.__OperationFinalStart[idx], self.__OperationFinalEnd[idx])

			self.FullText = ReplaceBlock(text, startp, endp, self.__OperationBackUp[idx])

			FormatColor4Line(self, self.__OperationStart[idx], self.__OperationEnd[idx])
			
			if self.__Operation[idx] == _Operation.DELETE then			
				AdjustCursorPosition(self, self.__OperationStart[idx])
				
				self:HighlightText(self.__OperationStart[idx], self.__OperationStart[idx])
			elseif self.__Operation[idx] == _Operation.BACKSPACE then			
				AdjustCursorPosition(self, self.__OperationEnd[idx])
				
				self:HighlightText(self.__OperationEnd[idx], self.__OperationEnd[idx])
			else				
				AdjustCursorPosition(self, self.__OperationEnd[idx])

				self:HighlightText(self.__OperationStart[idx], self.__OperationEnd[idx])
			end

			self.__OperationIndex = self.__OperationIndex - 1
		end

		return self:Fire("OnOperationListChanged")
	end

	------------------------------------
	--- Whether the CodeEditor has operation can be redo.
	-- @name  CodeEditor:CanRedo
	-- @class function
	-- @return true if the CodeEditor can redo operations
	-- @usage  CodeEditor:CanRedo()
	------------------------------------
	function CanRedo(self)
		return self.__OperationIndex < self.__MaxOperationIndex
	end

	------------------------------------
	--- Redo the operation.
	-- @name  CodeEditor:Redo
	-- @class function
	-- @usage  CodeEditor:Redo()
	------------------------------------
	function Redo(self)
		if self.__OperationIndex < self.__MaxOperationIndex then
			local idx = self.__OperationIndex + 1
			local text = self.FullText

			local startp, endp = GetLines(text, self.__OperationStart[idx], self.__OperationEnd[idx])

			self.FullText = ReplaceBlock(text, startp, endp, self.__OperationData[idx])

			FormatColor4Line(self, self.__OperationFinalStart[idx], self.__OperationFinalEnd[idx])
			
			AdjustCursorPosition(self, self.__OperationFinalEnd[idx])

			self:HighlightText(self.__OperationFinalEnd[idx], self.__OperationFinalEnd[idx])

			self.__OperationIndex = self.__OperationIndex + 1
		end

		return self:Fire("OnOperationListChanged")
	end

	------------------------------------
	--- Reset the CodeEditor's modified state
	-- @name  CodeEditor:ResetModifiedState
	-- @class function
	-- @usage  CodeEditor:ResetModifiedState()
	------------------------------------
	function ResetModifiedState(self)
		self.__DefaultText = self.FullText
	end

	------------------------------------
	--- Whether the CodeEditor is modified
	-- @name  CodeEditor:IsModified
	-- @class function
	-- @return true if the CodeEditor is modified
	-- @usage  CodeEditor:IsModified()
	------------------------------------
	function IsModified(self)
		return self.__DefaultText ~= self.FullText
	end

	function ShowCodeHelper(self, list)
		if type(list) == "table" and #list > 0 then
			_CodeSmartHelper.__CodeEditor = self
			_CodeSmartHelper.Visible = false
			_CodeSmartHelper:ClearAllPoints()
			_CodeSmartHelper:SetPoint("TOPLEFT", self, "TOPLEFT", self.__X+6, -self.__Y)

			_List.Items = list

			_CodeSmartHelper.Visible = true
		end
	end

	function IncludeCode(self, code)
		if type(code) == "string" and code ~= "" then
			FormatAll(code, self, true)
		end
	end
	
	function ResetEnviroment(self)
		self.Enviroment = nil
		wipe(self.__Definition)
	end
	
	------------------------------------------------------
	-- Property
	------------------------------------------------------
	property "TabWidth" {
		Get = function(self)
			return self.__TabWidth or _TabWidth
		end,
		Set = function(self, tabWidth)
			if tabWidth and tabWidth < 1 then
				error("The TabWidth must be greater than 0.", 2)
			end

			self.__TabWidth = tabWidth and floor(tabWidth)

			-- UpdateText
			self:SetText(self.FullText)
		end,
		Type = System.Number + nil,
	}

	property "DefaultColor" {
		Get = function(self)
			return self.__DefaultColor or _DefaultColor
		end,
		Set = function(self, color)
			self.__DefaultColor = color

			-- UpdateText
			self:SetText(self.FullText)
		end,
		Type = System.Widget.ColorType + nil,
	}

	property "CommentColor" {
		Get = function(self)
			return self.__CommentColor or _CommentColor
		end,
		Set = function(self, color)
			self.__CommentColor = color

			-- UpdateText
			self:SetText(self.FullText)
		end,
		Type = System.Widget.ColorType + nil,
	}

	property "StringColor" {
		Get = function(self)
			return self.__StringColor or _StringColor
		end,
		Set = function(self, color)
			self.__StringColor = color

			-- UpdateText
			self:SetText(self.FullText)
		end,
		Type = System.Widget.ColorType + nil,
	}

	property "NumberColor" {
		Get = function(self)
			return self.__NumberColor or _NumberColor
		end,
		Set = function(self, color)
			self.__NumberColor = color

			-- UpdateText
			self:SetText(self.FullText)
		end,
		Type = System.Widget.ColorType + nil,
	}

	property "InstructionColor" {
		Get = function(self)
			return self.__InstructionColor or _InstructionColor
		end,
		Set = function(self, color)
			self.__InstructionColor = color

			-- UpdateText
			self:SetText(self.FullText)
		end,
		Type = System.Widget.ColorType + nil,
	}

	property "FullText" {
		Set = function(self, text)
			MultiLineTextBox.SetText(self, text)
		end,

		Get = function(self)
			return MultiLineTextBox.GetText(self)
		end,

		Type = String,
	}

	property "Enviroment" {
		Get = function(self)
			return self.__Env
		end,
		
		Set = function(self, env)
			self.__Env = env or _G
		end,
		
		Type = Table + nil,
	}
	
	------------------------------------------------------
	-- Script Handlers
	------------------------------------------------------
	local function OnEditFocusGained(self)
		_KeyScan.FocusEditor = self
		_KeyScan.Visible = true
	end

	local function OnEditFocusLost(self)
		if _KeyScan.FocusEditor == self then
			_KeyScan.FocusEditor = nil
			_KeyScan.Visible = false
		end
	end

	local function OnMouseDown(self)
		EndPrevKey(self)

		SaveOperation(self)

		self.__MouseDownCur = self.CursorPosition

		if IsShiftKeyDown() then
			self.__MouseDownShift = true
			self.__MouseDownTime = nil
		else
			self.__MouseDownShift = false
			if self.__MouseDownTime and GetTime() - self.__MouseDownTime < _DBL_CLK_CHK then
				-- mean double click
				self.__DBLCLKSELTEXT = true
				self.__OldCursorPosition = nil
			else
				self.__MouseDownTime = GetTime()
			end
		end
	end

	local function OnMouseUp(self)
		self.__MouseDownCur = nil
		self.__MouseDownShift = nil
		if self.__MouseDownTime and GetTime() - self.__MouseDownTime < _DBL_CLK_CHK then
			self.__MouseDownTime = GetTime()
		else
			self.__MouseDownTime = nil
		end
	end

	local function OnEnterPressed(self)
		local cursorPos = self.CursorPosition
		local text = self.FullText
		local lstartp, lendp, lstr = GetLines(text, cursorPos - 1)
		local _, len, indent, startp, endp, str, lprevIndent, oprevIndent, prevIndent, lindent, llen

		lstr = lstr or ""

		_, llen = lstr:find("^%s+")

		llen = llen or 0

		lstr, lindent, lprevIndent = FormatIndent(lstr:sub(llen+1, -1), self.TabWidth)

		oprevIndent = lprevIndent

		if lprevIndent == 0 then
			if lindent == 0 then
				self:Insert(strrep(" ", llen))
			elseif lindent > 0 then
				self:Insert(strrep(" ", floor(llen/self.TabWidth)*self.TabWidth + self.TabWidth * lindent))
			end
		else
			startp, endp, str, len = lstartp, lendp, lstr, llen

			while startp > 1 do
				startp, endp, str = GetLines(text, startp - 2)

				if startp < endp then
					_, len = str:find("^%s+")

					len = len or 0

					if len < str:len() then
						str, indent, prevIndent = FormatIndent(str:sub(len+1, -1), self.TabWidth)

						lprevIndent = lprevIndent - indent

						if lprevIndent <= 0 then
							break
						end
					end
				end
			end

			if lprevIndent <= 0 then
				lstr = strrep(" ", floor(len / self.TabWidth) * self.TabWidth) .. lstr
				self.FullText = ReplaceBlock(text, lstartp, lendp, lstr)
				AdjustCursorPosition(self, lstartp + lstr:len())
				self:Insert(strrep(" ", floor(len / self.TabWidth) * self.TabWidth + self.TabWidth * (lindent + oprevIndent)))
			else
				self.FullText = ReplaceBlock(text, lstartp, lendp, lstr)
				AdjustCursorPosition(self, lstartp + lstr:len())
				self:Insert(strrep(" ", self.TabWidth * (lindent + oprevIndent)))
			end
		end

		return FormatColor4Line(self, lstartp)
	end

	local function OnTabPressed(self)
		local text = self.FullText

		if self.__HighlightTextStart == 0 and self.__HighlightTextEnd == self.FullText:len() then
			self:SetText(text)
			return AdjustCursorPosition(self, 0)
		end

		local startp, endp, str, lineBreak
		local shiftDown = IsShiftKeyDown()
		local cursorPos = self.CursorPosition

		if self.__HighlightTextStart and self.__HighlightTextEnd and self.__HighlightTextEnd > self.__HighlightTextStart then
			startp, endp, str = GetLines(text, self.__HighlightTextStart, self.__HighlightTextEnd)

			if str:find("\n") then
				lineBreak = "\n"
			elseif str:find("\r") then
				lineBreak = "\r"
			else
				lineBreak = false
			end

			if lineBreak then
				if shiftDown then
					str = str:gsub("[^".. lineBreak .."]+", _ShiftIndentFunc[self.TabWidth])
				else
					str = str:gsub("[^".. lineBreak .."]+", _IndentFunc[self.TabWidth])
				end

				self.FullText = ReplaceBlock(text, startp, endp, str)

				AdjustCursorPosition(self, startp + str:len())

				-- keep sleep here.
				Threading.Sleep(_WAITTIME)

				self:HighlightText(startp - 1, startp + str:len())

				return
			else
				if shiftDown then
					cursorPos = startp - 1 + floor((cursorPos - startp) / self.TabWidth) * self.TabWidth

					return AdjustCursorPosition(self, cursorPos)
				else
					cursorPos = self.TabWidth - (self.__HighlightTextStart - startp) % self.TabWidth

					str = str:sub(1, self.__HighlightTextStart - startp) .. strrep(" ", cursorPos) .. str:sub(self.__HighlightTextEnd + 2 - startp, -1)

					self.FullText = ReplaceBlock(text, startp, endp, str)

					cursorPos = self.__HighlightTextStart + cursorPos - 1

					return AdjustCursorPosition(self, cursorPos)
				end
			end
		else
			startp, endp, str = GetLines(text, cursorPos)

			if shiftDown then
				local _, len = str:find("^%s+")

				if len and len > 0 then
					if startp + len - 1 >= cursorPos then
						str = strrep(" ", len - self.TabWidth) .. str:sub(len + 1, -1)

						self.FullText = ReplaceBlock(text, startp, endp, str)

						if cursorPos - self.TabWidth >= startp - 1 then
							AdjustCursorPosition(self, cursorPos - self.TabWidth)
						else
							AdjustCursorPosition(self, startp - 1)
						end

						return
					else
						cursorPos = startp - 1 + floor((cursorPos - startp) / self.TabWidth) * self.TabWidth

						AdjustCursorPosition(self, cursorPos)

						return
					end
				end
			else
				local len = self.TabWidth - (cursorPos - startp + 1) % self.TabWidth

				str = str:sub(1, cursorPos - startp + 1) .. strrep(" ", len) .. str:sub(cursorPos - startp + 2, -1)

				self.FullText = ReplaceBlock(text, startp, endp, str)

				AdjustCursorPosition(self, cursorPos + len)
			end
		end
	end

	local function OnChar(self, char)
		if self.__InPasting or not self:HasFocus() or not char then
			return
		end

		self.__InCharComposition = nil

		-- Keep change to operation list
		if char == "." then
			-- check property
			local obj = GetPrevObject(self)
		elseif char == ":" then
			-- check method
			local obj = GetPrevObject(self)
		else
			-- Color the line
			return FormatColor4Line(self)
		end
	end

	local function OnCharComposition(self, text)
		self.__InCharComposition = true
	end

	local function OnCursorChanged(self, x, y, w, h)
		self.__X = x
		self.__Y = y

		if self.__InCharComposition then return end

		local cursorPos = self.CursorPosition

		if cursorPos == self.__OldCursorPosition then
			return
		end

		self.__OldCursorPosition = cursorPos

		if self.__InPasting then
			self.__InPasting = nil
			return FormatColor4Line(self, self.__HighlightTextStart)
		end

		if self.__DBLCLKSELTEXT then
			local str = self.FullText
			local startp, _, line = GetLines(str, cursorPos)
			local prevToken, prevPos

			local token, nextPos = nextToken(str, startp)

			while token and nextPos <= cursorPos do
				prevToken, prevPos = token, startp

				startp = nextPos
				token, nextPos = nextToken(str, startp)
			end

			if token and nextPos > cursorPos then
				if prevToken == _Token.COLORCODE_START then
					startp = prevPos
				end

				AdjustCursorPosition(self, nextPos - 1)

				self:HighlightText(startp - 1, nextPos - 1)
			end

			self.__DBLCLKSELTEXT = nil

			return
		end

		if self.__MouseDownShift == false then
			-- First CursorChanged after mouse down if not press shift
			self:HighlightText(cursorPos, cursorPos)

			self.__MouseDownCur = cursorPos
			self.__MouseDownShift = nil

			return
		end

		if self.__MouseDownCur then
			if self.__MouseDownCur == cursorPos then
				return
			end

			-- Hightlight all
			if self.__HighlightTextStart and self.__HighlightTextEnd and self.__HighlightTextStart ~= self.__HighlightTextEnd then
				if self.__HighlightTextStart == self.__MouseDownCur then
					self:HighlightText(cursorPos, self.__HighlightTextEnd)
				elseif self.__HighlightTextEnd == self.__MouseDownCur then
					self:HighlightText(cursorPos, self.__HighlightTextStart)
				else
					self:HighlightText(self.__MouseDownCur, cursorPos)
				end
			else
				self:HighlightText(self.__MouseDownCur, cursorPos)
			end

			self.__MouseDownCur = cursorPos

			return
		end

		if self.__BACKSPACE then
			return
		end

		if self.__DELETE then
			return
		end

		if self.__SKIPCURCHG then
			if tonumber(self.__SKIPCURCHG) then
				if self.__HighlightTextStart and self.__HighlightTextEnd and self.__HighlightTextStart ~= self.__HighlightTextEnd then
					if self.__HighlightTextStart == self.__SKIPCURCHG then
						self:HighlightText(cursorPos, self.__HighlightTextEnd)
					elseif self.__HighlightTextEnd == self.__SKIPCURCHG then
						self:HighlightText(cursorPos, self.__HighlightTextStart)
					else
						self:HighlightText(self.__SKIPCURCHG, cursorPos)
					end
				else
					self:HighlightText(self.__SKIPCURCHG, cursorPos)
				end
			end

			if not self.__SKIPCURCHGARROW then
				self.__SKIPCURCHG = nil
			else
				self.__SKIPCURCHG = cursorPos
			end

			return
		end

		-- CursorChanged not triggered by mouse event
		return self:HighlightText(cursorPos, cursorPos)
	end

	local function OnTextChanged(self, isUserInput)
		if not isUserInput then return end

		if self.__DELETE and _KeyScan.FocusEditor == self then
			if _CheckTimer.Interval == 0 then
				_CheckTimer.Interval = 0.8
			else
				_CheckTimer.Interval = 0.3
			end
		end

		if self.__BACKSPACE and _KeyScan.FocusEditor == self then
			if _CheckTimer.Interval == 0 then
				_CheckTimer.Interval = 0.8
			else
				_CheckTimer.Interval = 0.3
			end
		end
	end
	
	local function Thread_FindText(self)
		local searchText = IGAS:MsgBox(L"Please input the search content", "ic")
		
		self:SetFocus()
		
		searchText = searchText and strtrim(searchText)
		
		if searchText and searchText ~= "" then
			-- Prepare the search
			self.__InSearch = searchText
			self.__InSearch4Color = FormatColor(searchText, self)
			
			if self.__InSearch4Color:sub(1, 2) == "|c" then
				self.__InSearch4Color = self.__InSearch4Color:sub(11, -1)
			end
			
			if self.__InSearch4Color:sub(-2, -1) == "|r" then
				self.__InSearch4Color = self.__InSearch4Color:sub(1, -3)
			end
			
			Search2Next(self)
		end
	end
	
	local function Thread_GoLine(self)
		local goLine = IGAS:MsgBox(L"Please input the line number", "ic")
		
		self:SetFocus()
		
		goLine = goLine and tonumber(goLine)
		
		if goLine and goLine >= 1 then
			GoLineByNo(self, floor(goLine))
		end
	end
	
	function _KeyScan:OnKeyDown(key)
		if _SkipKey[key] then return end

		if self.FocusEditor and key then
			local editor = self.FocusEditor
			local cursorPos = editor.CursorPosition

			EndPrevKey(editor)

			local oper = _KEY_OPER[key]

			if oper then
				if oper == _Operation.CHANGE_CURSOR then

					if key == "PAGEUP" then
						local text = editor.FullText
						local skipLine = floor(editor.Height / editor.ValueStep)
						local startp, endp, _, line = GetPrevLinesByReturn(text, cursorPos, skipLine)

						if line == 0 then
							return
						end

						SaveOperation(editor)

						if editor.Value > editor.Height then
							editor.Value = editor.Value - editor.Height
						else
							editor.Value = 0
						end

						if IsShiftKeyDown() then
							editor.__SKIPCURCHG = cursorPos
						end

						editor.CursorPosition = GetCursorPosByOffset(text, startp, GetOffsetByCursorPos(text, nil, cursorPos))

						return
					end

					if key == "PAGEDOWN" then
						local text = editor.FullText
						local skipLine = floor(editor.Height / editor.ValueStep)
						local startp, endp, _, line = GetLinesByReturn(text, cursorPos, skipLine)

						if line == 0 then
							return
						end

						SaveOperation(editor)

						local maxValue = editor.Container.Height - editor.Height

						if editor.Value + editor.Height < maxValue then
							editor.Value = editor.Value + editor.Height
						else
							editor.Value = maxValue
						end

						if IsShiftKeyDown() then
							editor.__SKIPCURCHG = cursorPos
						end

						editor.CursorPosition = GetCursorPosByOffset(text, endp, GetOffsetByCursorPos(text, nil, cursorPos))

						return
					end

					if key == "HOME" then
						local text = editor.FullText
						local startp, endp = GetLines(text, cursorPos)
						local byte

						if startp - 1 == cursorPos then
							return
						end

						SaveOperation(editor)

						if IsShiftKeyDown() then
							editor.__SKIPCURCHG = cursorPos
						end

						return
					end

					if key == "END" then
						local startp, endp = GetLines(editor.FullText, cursorPos)

						if endp == cursorPos then
							return
						end

						SaveOperation(editor)

						if IsShiftKeyDown() then
							editor.__SKIPCURCHG = cursorPos
						end

						return
					end

					if key == "UP" then
						local _, _, _, line = GetPrevLinesByReturn(editor.FullText, cursorPos, 1)

						if line > 0 then
							SaveOperation(editor)

							if IsShiftKeyDown() then
								editor.__SKIPCURCHG = cursorPos
								editor.__SKIPCURCHGARROW = true
							end
						end

						return
					end

					if key == "DOWN" then
						local _, _, _, line = GetLinesByReturn(editor.FullText, cursorPos, 1)

						if line > 0 then
							SaveOperation(editor)

							if IsShiftKeyDown()  then
								editor.__SKIPCURCHG = cursorPos
								editor.__SKIPCURCHGARROW = true
							end
						end

						return
					end

					if key == "RIGHT" then
						if cursorPos < editor.FullText:len() then
							SaveOperation(editor)

							if IsShiftKeyDown() then
								editor.__SKIPCURCHG = cursorPos
								editor.__SKIPCURCHGARROW = true
							end
						end

						return
					end

					if key == "LEFT" then
						if cursorPos > 0 then
							SaveOperation(editor)

							if IsShiftKeyDown() then
								editor.__SKIPCURCHG = cursorPos
								editor.__SKIPCURCHGARROW = true
							end
						end

						return
					end
				end

				if key == "TAB" then
					editor.__INPUTTAB = true
					return NewOperation(editor, _Operation.INPUTTAB)
				end

				if key == "DELETE" then
					if not editor.__DELETE and not IsShiftKeyDown() and (editor.__HighlightTextStart ~= editor.__HighlightTextEnd or cursorPos < editor.FullText:len()) then
						editor.__DELETE = true
						return NewOperation(editor, _Operation.DELETE)
					end
					return
				end

				if key == "BACKSPACE" then
					if cursorPos > 0 then
						editor.__BACKSPACE = cursorPos
						return NewOperation(editor, _Operation.BACKSPACE)
					end
					return
				end

				if key == "ENTER" then
					editor.__SKIPCURCHG = true
					return NewOperation(editor, _Operation.ENTER)
				end
			end

			if key:find("^F%d+") == 1 then
				if key == "F3" and editor.__InSearch then
					-- Continue Search
					Search2Next(editor)
				end
				
				return editor:Fire("OnFunctionKey", key)
			end

			-- Don't consider multi-modified keys
			if IsShiftKeyDown() then
				-- shift+
			elseif IsAltKeyDown() then
				-- alt+
				return
			elseif IsControlKeyDown() then
				if key == "A" then
					editor:HighlightText()
					return
				elseif key == "V" then
					editor.__InPasting = true
					return NewOperation(editor, _Operation.PASTE)
				elseif key == "C" then
					-- do nothing
					return
				elseif key == "Z" then
					return editor:Undo()
				elseif key == "Y" then
					return editor:Redo()
				elseif key == "S" then
					return editor:Fire("OnSave")
				elseif key == "X" then
					if editor.__HighlightTextStart ~= editor.__HighlightTextEnd then
						NewOperation(editor, _Operation.CUT)
					end
					return
				elseif key == "F" then
					_Thread.Thread = Thread_FindText
					return _Thread(editor)
				elseif key == "G" then
					_Thread.Thread = Thread_GoLine
					return _Thread(editor)
				else
					return
				end
			end

			return NewOperation(editor, _Operation.INPUTCHAR)
		end
	end

	function _CheckTimer:OnTimer()
		local editor = _KeyScan.FocusEditor

		_CheckTimer.Interval = 0

		if editor then
			local cursorPos = editor.CursorPosition

			if editor.__DELETE then
				editor.__DELETE = nil

				return FormatColor4Line(editor)
			end

			if editor.__BACKSPACE then
				--[[
				local text = editor.FullText
				local startp, endp, str = GetLines(text, cursorPos)

				local _, len = str:find("^%s+")

				if len and len > 1 and startp + len - 1 >= cursorPos then
					if len % editor.TabWidth == 0 then
						return
					end

					str = str:sub(len + 1, -1)
					len = floor(len/editor.TabWidth) * editor.TabWidth

					str = strrep(" ", len) .. str

					editor.FullText = ReplaceBlock(text, startp, endp, str)

					return AdjustCursorPosition(editor, startp - 1 + len)
				end --]]

				editor.__BACKSPACE = nil
				
				return FormatColor4Line(editor)
			end
		end
	end

	function _List:OnItemChoosed(key, text)
		_CodeSmartHelper.Text = text
	end

	function _List:OnItemDoubleClick(key, text)
		_CodeSmartHelper.Text = text
		_CodeSmartHelper:Fire("OnEnterPressed")
	end

	function _CodeSmartHelper:OnShow()
		self.Text = ""
		_List:ClearAllPoints()
		if self:GetBottom() - _List.Height > 0 then
			_List:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
			_List:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT")
		else
			_List:SetPoint("BOTTOMLEFT", self, "TOPLEFT")
			_List:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT")
		end
		_List:Refresh()
		_List:SelectItemByValue(nil)
		self:SetFocus()
	end

	function _CodeSmartHelper:OnCharComposition(char)
		local list = _List.Items
		local text = self:GetText()
		local i, name

		if not text or text == "" then
			return
		end

		i, name = next(list, GetIndex(list, text))
		while i do
			if strfind(strupper(name), strupper(text), 1, 1) == 1 then
				_List:SelectItemByIndex(i)
				self:SetText(name)
				if (self:IsInIMECompositionMode()) then
					self:HighlightText(strlen(text) - strlen(char), -1)
				else
					self:HighlightText(strlen(text), -1)
				end
				return
			end
			i, name = next(list, i)
		end
	end

	function _CodeSmartHelper:OnChar(char)
		if ( not self:IsInIMECompositionMode() ) then
			self:Fire("OnCharComposition", char)
		end
	end

	function _CodeSmartHelper:OnEditFocusGained()
		self:HighlightText()
	end

	function _CodeSmartHelper:OnEditFocusLost()
		self.Visible = false
	end

	function _CodeSmartHelper:OnTabPressed()
		self:Fire("OnEnterPressed")
	end

	function _CodeSmartHelper:OnEnterPressed()
		self:HighlightText(0, 0)

		self.__CodeEditor:SetFocus()
		self.__CodeEditor:Insert(self.Text)
		self.Visible = false
		self.__CodeEditor = nil
	end

	function _CodeSmartHelper:OnEscapePressed()
		self:HighlightText(0, 0)
		self.Text = ""
		self.__CodeEditor:SetFocus()
		self.Visible = false
		self.__CodeEditor = nil
	end

	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
    function CodeEditor(name, parent)
		local frame = MultiLineTextBox(name, parent)

		-- Setup
		frame:ClearFocus()
	
		-- Script Handlers
		frame.OnTabPressed = frame.OnTabPressed + OnTabPressed
		frame.OnEditFocusGained = frame.OnEditFocusGained + OnEditFocusGained
		frame.OnEditFocusLost = frame.OnEditFocusLost + OnEditFocusLost
		frame.OnEnterPressed = frame.OnEnterPressed + OnEnterPressed
		frame.OnCursorChanged = frame.OnCursorChanged + OnCursorChanged
		frame.OnMouseDown = frame.OnMouseDown + OnMouseDown
		frame.OnMouseUp = frame.OnMouseUp + OnMouseUp
		frame.OnChar = frame.OnChar + OnChar
		frame.OnCharComposition = frame.OnCharComposition + OnCharComposition
		frame.OnTextChanged = frame.OnTextChanged + OnTextChanged

		frame:ActiveThread("OnTabPressed")

		-- Operation Keep List
		frame.__Operation = {}

		frame.__OperationStart = {}
		frame.__OperationEnd = {}
		frame.__OperationBackUp = {}

		frame.__OperationFinalStart = {}
		frame.__OperationFinalEnd = {}
		frame.__OperationData = {}

		frame.__OperationIndex = 0
		frame.__MaxOperationIndex = 0

		-- Enviroment
		frame.__Env = _G
		frame.__Definition = {}
		
		return frame
	end
endclass "CodeEditor"
